%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Generate Impulse Response
% 9 November 2006, Release 0.85
%
% Comment: This function applies the spatial filtering, does the discrete
% time to continuous time conversion, and then generates the actual complex baseband impulse
% responses to be used for proposal analysis.
%
% Input:
%
% Output:
%
% Internal Variables:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp(' ')
disp('--> Running Generate Impulse Response')
disp(' This routine generates a complex baseband impulse response')

SampFreq = input(' Input Sample Frequency (Gsps): ');
SampFreq=SampFreq*1e9;

AntBeamWidth = 0;
while (AntBeamWidth>360 || AntBeamWidth<1)
    AntBeamWidth = input(' Input Antenna Beam Width: (1 to 360 degs) ');
end
AntBeamWidth=AntBeamWidth*pi/180; % convert to radians

AntBeamAng = -181;
while (AntBeamAng>180 || AntBeamAng<-180)
    AntBeamAng = input(' Input Antenna Pointing Angle: (-180 to 180 degs): ');
end
AntBeamAng=AntBeamAng*pi/180; % convert to radians

ts = 1/SampFreq;  % sampling frequency

% derive the impulse response

load ChanNum
cd(strcat('CM',num2str(ChanNum)))
load FullVectors
load ImpInfoStuff
load SortedVectors
cd ..

% Generate complex continuous time impulse response
Phase = rand(size(SortedAmp));                                                  % Random phase
ImpCt = sqrt(SortedAmp) .* exp(2*pi*1i*Phase);                                % Instantaneous impulse response

%%%%%%%%%%%%%%%%%%  FIX  %%%%%%%%%%%%%%%%%%%%%%%%%%%

TimeCt=SortedTime;
t0=zeros(1,NumChannels);
np=NumRays*ones(1,NumChannels);
Ang=SortedAng;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Circle=(-pi:0.01:pi);
radius=max(max(abs(ImpCt)));
figure(1)
hold on
for ChanIndex=1:NumChannels
    temp1=mod(ChanIndex,7);
    if temp1==1
        color='b';
    elseif temp1==2
        color='g';
    elseif temp1==3
        color='r';
    elseif temp1==4
        color='c';
    elseif temp1==5
        color='m';
    elseif temp1==6
        color='y';
    else
        color='k';
    end
    polar(Ang(:,ChanIndex),abs(ImpCt(:,ChanIndex)),color)
end
polar(Circle,radius*ones(size(Circle)),'k--')
title('Ray Polar Plot before Spatial Filtering')
hold off

%%%%%%%%%%%%% SPATIAL FILTERING - EXAMPLE IS DONE HERE %%%%%%%%%%%%%%%%%%%%

% prepare the spatial filter edges

RhEdge = mod( AntBeamAng - AntBeamWidth/2 + pi , 2*pi ) - pi;
LhEdge = mod( AntBeamAng + AntBeamWidth/2 + pi , 2*pi ) - pi;

for cn=1:NumChannels
    for index=1:length(ImpCt(:,1))
        if (RhEdge-LhEdge)<0
            if ((Ang(index,cn)>LhEdge) || (Ang(index,cn)<RhEdge)) % ant beam width about bore sight
                ImpCt(index,cn)=0; % spatial filter
            end
        else
            if ((Ang(index,cn)>LhEdge) && (Ang(index,cn)<RhEdge)) % ant beam width about bore sight
                ImpCt(index,cn)=0; % spatial filter
            end
        end
    end
end

figure(2)
hold on
for ChanIndex=1:NumChannels
    temp1=mod(ChanIndex,7);
    if temp1==1
        color='b';
    elseif temp1==2
        color='g';
    elseif temp1==3
        color='r';
    elseif temp1==4
        color='c';
    elseif temp1==5
        color='m';
    elseif temp1==6
        color='y';
    else
        color='k';
    end
    polar(Ang(:,ChanIndex),abs(ImpCt(:,ChanIndex)),color)
end
polar([AntBeamWidth/2+AntBeamAng AntBeamWidth/2+AntBeamAng],[radius 0],'r--')
polar([-AntBeamWidth/2+AntBeamAng -AntBeamWidth/2+AntBeamAng],[radius 0],'r--')
polar(Circle,radius*ones(size(Circle)),'k--')
title('Ray Polar Plot after Spatial Filtering')
hold off

% now reduce continuous-time result to a discrete-time result
[ImpDt] = ct2dt( ImpCt, TimeCt, NumChannels, ts );

% normal the energy in each realization to a value of 1
NothingLeft=zeros(NumChannels,1);
for ChanIndex=1:NumChannels
    SpatialFilteredEnergy(ChanIndex)=sum(abs(ImpDt(:,ChanIndex)).^2);
    if SpatialFilteredEnergy(ChanIndex)~=0
        ImpDt(:,ChanIndex)=ImpDt(:,ChanIndex)/sqrt(sum(abs(ImpDt(:,ChanIndex)).^2));
    else
        NothingLeft(ChanIndex)=1;
    end
end

if sum(NothingLeft)==NumChannels
    beep; disp(' ')
    disp('SORRY - YOU SPATIALLY FILTERED OUT EVERYTHING!')
    disp('THERE IS NO SIGNAL LEFT IN ANY REALIZATION!')
    disp('TERMINATING NOW - BYE')
    load xgong
    p=audioplayer(y, Fs);
    play(p)
    cd(strcat('CM',num2str(ChanNum)))
    delete IR_real.xls 
    delete IR_imag.xls
    delete ImpResp.mat
    cd ..
    return
end

% truncate impulse response to the -40 dB point - FIX THIS
z_max=max(max(abs(ImpDt)));
for index_cn=1:NumChannels
    IM_done=0;
    for index=length(ImpDt):-1:1
        if IM_done==0
            if abs(ImpDt(index,index_cn))>z_max/1e2 % FIX THIS on TSV model
                index_max(index_cn)=index;
                IM_done=1;
            end
        end
    end
end
ImpDtTrunc=ImpDt(1:max(index_max),:);

h_len = length(ImpDtTrunc(:,1));
if h_len>1
    TimeDt = (0:(h_len-1)) * ts;  % for use in computing excess & RMS delays
else
    TimeDt = ts; % single point in the impulse response
end

cd(strcat('CM',num2str(ChanNum)))
save ImpResp ImpDtTrunc TimeDt t0 NumChannels NothingLeft
cd ..

%********************************************************************
% Testing and ploting
%********************************************************************

% channel energy
kk=0;
for ChanIndex=1:NumChannels
    if NothingLeft(ChanIndex)==0
        kk=kk+1; % artifical counter so nulled channels don't matter
        [Row,Col]=size(ImpDtTrunc);
        if Row>1
            channel_energy(kk) = sum(abs(ImpDtTrunc(:,ChanIndex)).^2); 
            if channel_energy(kk)==0
                kk
                ChanIndex
            end
        else
            channel_energy(kk) = abs(ImpDtTrunc(ChanIndex)).^2; 
        end
    end
end

% calculate metrics
excess_delay = zeros(1,NumChannels);
RMS_delay = zeros(1,NumChannels);
num_sig_paths = zeros(1,NumChannels);
num_sig_e_paths = zeros(1,NumChannels);

kk=0;
for k=1:NumChannels
    if NothingLeft(k)==0
       kk=kk+1; % artifical counter so nulled channels don't matter
       % determine excess delay and RMS delay
       sq_h = abs(ImpDtTrunc(:,k)).^2 / channel_energy(kk);
       t_norm = TimeDt - t0(k);  % remove the randomized arrival time of first cluster
       excess_delay(kk) = t_norm * sq_h;
       RMS_delay(kk) = sqrt( ((t_norm-excess_delay(kk)).^2) * sq_h );

       % determine number of significant paths (paths within 10 dB from peak)
       threshold_dB = -10;   % dB
       temp_h = abs(ImpDtTrunc(:,k));
       temp_thresh = 10^(threshold_dB/20) * max(temp_h);
       num_sig_paths(kk) = sum(temp_h > temp_thresh);

       % determine number of sig. paths (captures x % of energy in channel)
       x = 0.85;
       temp_sort = sort(temp_h.^2);  % sorted in ascending order of energy
       cum_energy = cumsum(temp_sort(end:-1:1));  % cumulative energy
       index_e = min(find(cum_energy >= x * cum_energy(end)));
       num_sig_e_paths(kk) = index_e;
    end
end

energy_mean = mean(10*log10(channel_energy));
energy_stddev = std(10*log10(channel_energy));
mean_excess_delay = mean(excess_delay);
mean_RMS_delay = mean(RMS_delay);
mean_sig_paths = mean(num_sig_paths);
mean_sig_e_paths = mean(num_sig_e_paths);

disp(' '); fprintf(1,'Model Characteristics\n');
fprintf(1,'  Mean delays: excess (tau_m) = %1.2f ns, RMS (tau_rms) = %1.2f ns\n', ...
    1e9*mean_excess_delay, 1e9*mean_RMS_delay);
fprintf(1,'  # paths: NP_10dB =  %.1f, NP_85%% = %.1f\n', ...
    mean_sig_paths, mean_sig_e_paths);
fprintf(1,'  Channel energy: mean = %.1f dB, std deviation = %.1f dB\n', ...
  energy_mean, energy_stddev);
fprintf(1,'  Channels Spatially Nulled: %.1f , Remaining Channels: %.1f\n', ...
  sum(NothingLeft), NumChannels-sum(NothingLeft));

figure(3)
MaxImp=max(max(abs(ImpDtTrunc)));
plot(TimeDt,20*log10(1e-2+abs(ImpDtTrunc/MaxImp))); grid on
axis tight
title('Impulse Response dB Magnitude')
xlabel('nS')
ylabel('dB')

figure(4)
subplot(2,1,1); plot(TimeDt,real(ImpDtTrunc)); title('real impulse response'); grid on
subplot(2,1,2); plot(TimeDt,imag(ImpDtTrunc)); title('imag impulse response'); grid on
xlabel('nS')

figure(5); clf; plot([1:NumChannels], excess_delay, 'b-', ...
  [1 NumChannels], mean_excess_delay*[1 1], 'r--' );
grid on
title('Excess delay (nS)')
xlabel('Channel number')

figure(6); clf; plot([1:NumChannels], RMS_delay, 'b-', ...
  [1 NumChannels], mean_RMS_delay*[1 1], 'r--' );
grid on
title('RMS delay (nS)')
xlabel('Channel number')

figure(7); clf; plot([1:NumChannels], num_sig_paths, 'b-', ...
  [1 NumChannels], mean_sig_paths*[1 1], 'r--');
grid on
title('Number of significant paths within 10 dB of peak')
xlabel('Channel number')

figure(8); clf; plot([1:NumChannels], num_sig_e_paths, 'b-', ...
  [1 NumChannels], mean_sig_e_paths*[1 1], 'r--');
grid on
title('Number of significant paths capturing > 85% energy')
xlabel('Channel number')

temp_average_power = sum((abs(ImpDtTrunc))'.*(abs(ImpDtTrunc))')/NumChannels;
temp_average_power = temp_average_power/max(temp_average_power);
average_decay_profile_dB = 10*log10(temp_average_power+1e-20);
figure(9); clf; plot(TimeDt,average_decay_profile_dB); grid on
axis([0 TimeDt(end) -60 0])
title('Average Power Decay Profile')
xlabel('Delay (nsec)')
ylabel('Average power (dB)')

figure(10)
stem(NothingLeft,'.')
title('Nulled Channels due to Spatial Filtering')
xlabel('Channel Number')
ylabel('Nulled Channel')
grid on

% store off the impulse responses in ASCII format
IR_real=[TimeDt' real(ImpDtTrunc)];
IR_imag=[TimeDt' real(ImpDtTrunc)];

disp(' ')
disp(strcat('Writing ASCII files IR_real.xls and IR_imag.xls to directory CM',num2str(ChanNum)))

cd(strcat('CM',num2str(ChanNum)))
save IR_real.xls IR_real -ASCII -TABS
save IR_imag.xls IR_imag -ASCII -TABS
cd ..

return; % end of program
